from pydantic import BaseModel


class CertifiedMunicipalityBase(BaseModel):
    """
    Certified Municipality  schema.
    """

    ugf: str
    town_hall_name: str
    address: str
    zip_code: str
    city_name: str
    phone_number: str
    website: str
    appointment_details: str
    service_opening_date: str
    label: str


class CertifiedMunicipalityCreate(CertifiedMunicipalityBase):
    """
    Certified Municipality create schema.
    """


class CertifiedMunicipalityUpdate(CertifiedMunicipalityBase):
    """
    Certified Municipality update schema.
    """


class CertifiedMunicipality(CertifiedMunicipalityBase):
    """
    Certified Municipality schema.
    """

    class Config:
        orm_mode = True



from typing import Any

from fastapi import APIRouter, Depends, File, HTTPException, Request, UploadFile
from fastapi.responses import StreamingResponse
from pydantic import Required
from slowapi import Limiter
from slowapi.util import get_remote_address
from sqlalchemy.orm import sessionmaker

from ...db.postgresdb_utils import get_database
from ...services.certified_municipality_service import (
    get_all_certified_municipalities,
    update_certified_municipality_table,
)
from ...services.meeting_point_service import get_all, update_meeting_points_table
from ..dependencies.auth_token import verify_internal_auth_token

router = APIRouter()

limiter = Limiter(key_func=get_remote_address)


@router.get(
    "/meetingPoints",
    responses={
        200: {
            "description": "Liste des meeting points en base de données",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "city_name": "Corme-Royal",
                            "id_editor": 5,
                            "editor_name_and_id": "RDV360100358",
                            "ugf": "17046",
                        },
                        {
                            "city_name": "Huelgoat",
                            "id_editor": 2,
                            "editor_name_and_id": "Synbird2036",
                            "ugf": "29014",
                        },
                    ]
                }
            },
        }
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
@limiter.limit("30/minute")
async def get_meeting_points(
    request: Request, session: sessionmaker = Depends(get_database)
) -> Any:
    """
    Récupère les données des meeting points en base de données.
    """
    try:
        return get_all(session)
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while getting meeting points : {str(e)}",
        )
    finally:
        session.close()


@router.post(
    "/updateMeetingPoints",
    responses={
        200: {
            "description": "Mise à jour des meeting points effectuée avec succès",
            "content": {
                "application/json": {
                    "example": {
                        "nb_meeting_points": 3043,
                        "created : ": "0",
                        "unchanged : ": "3043",
                    }
                }
            },
        }
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
async def update_meeting_points(
    request: Request,
    uploaded_file: UploadFile = File(
        default=Required, media_type="application/vnd.ms-excel"
    ),
    session: sessionmaker = Depends(get_database),
) -> Any:
    """
    <p>
        L’API permet à l'ANTS de mettre à jour la table des meeting points.
        Le fichier Excel doit être sous la forme suivante:
        <table class="table-bordered">
            <thead>
                <tr>
                    <th>UGF/th>
                    <th>editor_name_and_id</th>
                    <th>city_name</th>
                    <th>city_name</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>{Num UGF}</td>
                    <td>{Nom éditeur concaténé avec leur id meeting point}</td>
                    <td>{Nom de la ville}</td>
                    <td>{ID de l'éditeur}</td>
                </tr>
                <tr>
                    <td>{Num UGF}</td>
                    <td>{Nom éditeur concaténé avec leur id meeting point}</td>
                    <td>{Nom de la ville}</td>
                    <td>{ID de l'éditeur}</td>
                </tr>
            </tbody>
        </table>
        </br></br>Accès via un token de sécurité unique.
    </p>
    <p>
    Règle de gestion :</br>
    <ul>
        <li>Si l'ensemble ugf et editor_name_and_id n'existe pas: un meeting point est créé en base.</li>
        <li>Sinon on le comptabilise dans les inchangés.</li>
    </ul>
    </p>
    """
    try:
        return StreamingResponse(
            content=update_meeting_points_table(session, uploaded_file),
            media_type="application/json",
        )
    except Exception:
        raise HTTPException(
            status_code=500,
            detail="Error while updating meeting points",
        )
    finally:
        session.close()


@router.get(
    "/certifiedMunicipalities",
    responses={
        200: {
            "description": "Liste des mairies certifiées en base de données",
            "content": {
                "application/json": {
                    "example": [
                        {
                            "department_code": "69",
                            "department_name": "Rhône",
                            "municipalities": [
                                {
                                    "ugf": "69080",
                                    "town_hall_name": "Mairie de COLOMBIER SAUGNIEU",
                                    "address": "14 rue de la Mairie",
                                    "zip_code": "69124",
                                    "phone_number": "04 78 32 80 17",
                                    "website": "https://www.mairie-colombiersaugnieu.fr/demarches-administratives/en-mairie/",
                                    "city_name": "COLOMBIER SAUGNIEU",
                                    "appointment_details": "Lundi : 14h – 17h\nMardi et jeudi : 9h – 12h / 14h – 18h\nMercredi : 9h – 17h\nVendredi et samedi : 9h – 12h ",
                                    "service_opening_date": "2024-03-25",
                                    "label": "sans rendez-vous",
                                    "department_code": "69",
                                    "department_name": "Rhône",
                                    "cannonized_name": "colombiersaugnieu",
                                },
                                {
                                    "ugf": "69071",
                                    "town_hall_name": "Mairie de CHASSIEU",
                                    "address": "60 rue de la République",
                                    "zip_code": "69680",
                                    "phone_number": "04 72 05 44 00",
                                    "website": "https://www.chassieu.fr/",
                                    "city_name": "CHASSIEU",
                                    "appointment_details": "Du lundi au vendredi de 08h00 à 12h00 et de 13h00 à 17h00,\nle mardi de 08h00 à 12h00 et de 14h00 à 17h00\npendant les permanences de la direction (hors vacances scolaires) :\nle mardi de 17h00 à 19h00,\nle samedi de 09h00 à 12h00",
                                    "service_opening_date": "2024-03-11",
                                    "label": "sans rendez-vous",
                                    "department_code": "69",
                                    "department_name": "Rhône",
                                    "cannonized_name": "chassieu",
                                },
                            ],
                        },
                        {
                            "department_code": "92",
                            "department_name": "Hauts-de-Seine",
                            "municipalities": [
                                {
                                    "ugf": "92040",
                                    "town_hall_name": "Mairie Annexe de CLAMART",
                                    "address": "5, rue d’Auvergne",
                                    "zip_code": "92140",
                                    "phone_number": "01 46 62 35 35",
                                    "website": "https://www.clamart.fr",
                                    "city_name": "Clamart",
                                    "appointment_details": " Les lundis, mercredis, vendredis de 8h30 à 12h30 et de 13h30 à 17h00",
                                    "service_opening_date": "2024-03-11",
                                    "label": "sans rendez-vous",
                                    "department_code": "92",
                                    "department_name": "Hauts-de-Seine",
                                    "cannonized_name": "clamart",
                                },
                            ],
                        },
                    ]
                }
            },
        }
    },
)
async def get_certified_municipalities(
    request: Request, session: sessionmaker = Depends(get_database)
) -> Any:
    """
    Récupère les données des mairies certifiées en base de données.
    """
    try:
        return get_all_certified_municipalities(session)
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Error while getting certified municipality : {str(e)}",
        )
    finally:
        session.close()


@router.post(
    "/updateCertifiedMunicipalities",
    responses={
        200: {
            "description": "Mise à jour des mairies certifiées effectuée avec succès",
            "content": {
                "application/json": {
                    "example": {
                        "nb_certified_municipality": 12,
                        "created : ": "12",
                        "unchanged : ": "0",
                    }
                }
            },
        }
    },
    dependencies=[Depends(verify_internal_auth_token)],
)
async def update_certified_municipalities(
    request: Request,
    uploaded_file: UploadFile = File(
        default=Required, media_type="application/vnd.ms-excel"
    ),
    session: sessionmaker = Depends(get_database),
) -> Any:
    """
    <p>
        L’API permet à l'ANTS de mettre à jour la table des mairies certifiées.
        Le fichier Excel doit être sous la forme suivante:
        <table class="table-bordered">
            <thead>
                <tr>
                    <th>Nom de la mairie</th>
                    <th>UGF</th>
                    <th>Adresse</th>
                    <th>Code postal</th>
                    <th>Ville</th>
                    <th>Téléphone</th>
                    <th>Site internet</th>
                    <th>Modalités de rendez-vous</th>
                    <th>Date d'ouverture du service</th>
                    <th>Label</th>
                </tr>
            </thead>
            <tbody>
                <tr>
                    <td>Mairie de CHASSIEU</td>
                    <td>69071</td>
                    <td>60 rue de la République</td>
                    <td>69680</td>
                    <td>CHASSIEU</td>
                    <td>04 72 05 44 00</td>
                    <td>https://www.chassieu.fr/</td>
                    <td>Du lundi au vendredi de 08h00 à 12h00 et de 13h00 à 17h00...</td>
                    <td>11/03/2024</td>
                    <td>sans rendez-vous</td>
                </tr>
                <tr>
                    <td>Mairie principale de CLAMART</td>
                    <td>92072</td>
                    <td>1, place Maurice Gunsbourg</td>
                    <td>92140</td>
                    <td>Clamart</td>
                    <td>01 46 62 35 35</td>
                    <td>https://www.clamart.fr</td>
                    <td>les lundis, mercredis, jeudis de 8h30 à 12h30 ...</td>
                    <td>11/03/2024</td>
                    <td>sans rendez-vous</td>
                </tr>
            </tbody>
        </table>
        </br></br>Accès via un token de sécurité unique.
    </p>
    <p>
    Règle de gestion :</br>
    <ul>
        <li>Si l'ugf n'existe pas: une entrée est créé en base.</li>
        <li>Sinon on le comptabilise dans les inchangés.</li>
    </ul>
    Il est important que chaque champ soit renseigné avec le bon format (url commençant par https:// etc...).
    </p>
    """
    try:
        return StreamingResponse(
            content=update_certified_municipality_table(session, uploaded_file),
            media_type="application/json",
        )
    except Exception:
        raise HTTPException(
            status_code=500,
            detail="Error while updating meeting points",
        )
    finally:
        session.close()



import io
import json
import logging
from collections import defaultdict
from typing import List

from fastapi import UploadFile
from openpyxl import load_workbook
from sqlalchemy.orm import sessionmaker

from src.hubrdvmairie.models.certified_municipality import CertifiedMunicipality

from ..crud.crud_certified_municipality import certified_municipality as crud

logger = logging.getLogger(__name__)


def get_all(session) -> List[CertifiedMunicipality]:
    try:
        return crud.get_all(session)
    except Exception as e:
        logger.error("Error while getting all certified_municipality : %s", str(e))


def get_all_certified_municipalities(
    session: sessionmaker,
) -> List[CertifiedMunicipality]:
    municipalities = get_all(session)
    certified_municipalities_complete = []
    for certified_municipality in municipalities:
        complete = certified_municipality.complete()
        certified_municipalities_complete.append(complete)
    departments = defaultdict(list)

    for municipality in certified_municipalities_complete:
        department_code = municipality["department_code"]
        departments[department_code].append(municipality)

    departments_list = []
    for department_code, municipalities in departments.items():
        department_info = {
            "department_code": department_code,
            "department_name": municipalities[0]["department_name"],
            "municipalities": municipalities,
        }
        departments_list.append(department_info)
    departments_list = sorted(departments_list, key=lambda x: x["department_code"])
    return departments_list


async def update_certified_municipality_table(
    session: sessionmaker, uploaded_file: UploadFile
):
    certified_municipalities = await read_certified_municipality_from_file_streaming(
        uploaded_file
    )
    create_list = []
    unchanged_list = []
    nb_certified_municipalities = len(certified_municipalities)
    for certified_municipality in certified_municipalities:
        res = crud.save_or_update(session, obj_in=certified_municipality)
        if res[0] == "created":
            create_list.append(res[1])
        else:
            unchanged_list.append(res[1])

    yield json.dumps(
        {
            "nb_certified_municipality": nb_certified_municipalities,
            "created : ": str(len(create_list)),
            "unchanged : ": str(len(unchanged_list)),
        }
    )


async def read_certified_municipality_from_file_streaming(
    uploaded_file: UploadFile,
) -> CertifiedMunicipality:
    # read file depending on its type
    if uploaded_file.filename.endswith(".xlsx"):
        return await read_certified_municipality_file_streaming(uploaded_file)
    else:
        raise TypeError("Unknown file type : " + str(uploaded_file.filename))


async def read_certified_municipality_file_streaming(uploaded_file: UploadFile):
    certified_municipalities = set()

    file_content = await uploaded_file.read()
    xls_data = io.BytesIO(file_content)
    data = load_workbook(xls_data, read_only=True).active
    for ligne in data.iter_rows(min_row=2, max_row=data.max_row, values_only=True):
        ugf = str(ligne[1])
        town_hall_name = ligne[0]
        address = ligne[2]
        zip_code = str(ligne[3])
        city_name = ligne[4]
        phone_number = ligne[5]
        website = ligne[6]
        appointment_details = str(ligne[7])
        service_opening_date = ligne[8]
        label = ligne[9]

        certified_municipality = CertifiedMunicipality(
            ugf=ugf,
            town_hall_name=town_hall_name,
            address=address,
            zip_code=zip_code,
            city_name=city_name,
            phone_number=phone_number,
            website=website,
            appointment_details=appointment_details,
            service_opening_date=service_opening_date,
            label=label,
        )
        certified_municipalities.add(certified_municipality)
    return certified_municipalities
